/* agg - the news aggregator
 * Copyright (C) 2011 Andreas Waidler <arandes@programmers.at>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "text.h"
#include "layer.h"
#include "stack.h"

#define TEST(expr) test(#expr, expr)
void test(const char* expr, int res);

int  test_text_is_empty();
int  test_text_enable();
int  test_text_disable();
int  test_text_write();
int  test_text_clear();
int  test_text_append();
int  test_text_fill();

int  test_stack_set_top();
int  test_stack_next_prev();
int  test_stack_set_next();

int main()
{
	TEST(test_text_is_empty());
	TEST(test_text_enable());
	TEST(test_text_disable());
	TEST(test_text_write());
	TEST(test_text_clear());
	TEST(test_text_append());
	TEST(test_text_fill());

	TEST(test_stack_set_top());
	TEST(test_stack_next_prev());
	TEST(test_stack_set_next());

	return 0;
}

void test(const char* expr, int res)
{
	printf("  %-32s: %s\n", expr, res ? "OK" : "FAIL");
	if (!res) exit(1);
}



int test_text_is_empty()
{
	return strcmp("", text_get()) == 0;
}

int test_text_enable()
{
	text_enable();
	return 1;
}

int test_text_disable()
{
	text_disable();
	return 1;
}

int test_text_write()
{
	text_buffer("foobar", 3);
	return strcmp("foo", text_get()) == 0;
}

int test_text_clear()
{
	text_enable();
	if (!test_text_is_empty()) return 0;

	test_text_write();
	text_disable();
	return test_text_is_empty();
}

int test_text_append()
{
	text_buffer("foobar", 3);
	text_buffer("foobar", 3);
	return strcmp("foofoo", text_get()) == 0;
}

int test_text_fill()
{
	/* buffer must be >= text buffer. */
	#define BUFSIZE (1024 * 1024)
	char buf[BUFSIZE];
	memset(buf, ' ', BUFSIZE);
	text_buffer(buf, BUFSIZE);

	return strncmp(buf, text_get(), BUFSIZE) != 0;
}



int test_stack_set_top()
{
	struct Layer mock;
	mock.enter = (void (*)(const char*, const char**)) 0x7E57AB1E;
	mock.leave = (void (*)(const char*)) 0xCA11AB1E;

	stack_set(0, mock);
	return stack_top()->enter == mock.enter
	    && stack_top()->leave == mock.leave;
}

int test_stack_next_prev()
{
	struct Layer mock;
	mock.enter = (void (*)(const char*, const char**)) 0x7E57AB1E;
	mock.leave = (void (*)(const char*)) 0xCA11AB1E;

	/* Mock still set from method before. */

	stack_next();
	stack_prev();

	return stack_top()->enter == mock.enter
	    && stack_top()->leave == mock.leave;
}

int test_stack_set_next()
{
	struct Layer mock;
	mock.enter = (void (*)(const char*, const char**)) 0x7E57AB1E;
	mock.leave = (void (*)(const char*)) 0xCA11AB1E;

	stack_set(1, mock);
	stack_next();
	return stack_top()->enter == mock.enter
	    && stack_top()->leave == mock.leave;
}
